<?php

namespace Internetgalerie\IgCrmTemplate\Utility;

use Internetgalerie\IgCrmTemplate\Domain\Model\AddressInterface;
use Internetgalerie\IgCrmTemplate\Domain\Model\CssPagedMediaMarginBoxs;
use Internetgalerie\IgCrmTemplate\Domain\Model\TenantInterface;
use Internetgalerie\IgCrmTemplate\Domain\Model\TemplateObjectInterface;
use Internetgalerie\IgCrmTemplate\Domain\Model\TemplateLetter;
use Internetgalerie\IgCrmTemplate\Domain\Model\TemplatePageLayout;
use Internetgalerie\IgCrmTemplate\Domain\Repository\TemplateLetterRepository;
use Internetgalerie\IgCrmTemplate\Domain\Repository\TemplatePageLayoutRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3Fluid\Fluid\Core\Rendering\RenderingContextInterface;

/**
 * TemplateUtility
 */
class TemplateUtility implements \TYPO3\CMS\Core\SingletonInterface
{
    /**
     * tenant Uid
     */
    protected ?int $tenantUid = null;

    /**
     * crm language uir
     */
    protected ?int $languageUid = 0;


    /**
     * templateLetterRepository
     */
    protected ?TemplateLetterRepository $templateLetterRepository = null;

    /**
     * templatePageLayoutRepository
     */
    protected ?TemplatePageLayoutRepository $templatePageLayoutRepository = null;

 
    public function injectTemplateLetterRepository(TemplateLetterRepository $templateLetterRepository)
    {
        $this->templateLetterRepository = $templateLetterRepository;
    }

    public function injectTemplatePageLayoutRepository(TemplatePageLayoutRepository $templatePageLayoutRepository)
    {
        $this->templatePageLayoutRepository = $templatePageLayoutRepository;
    }

    /**                                                                                                          
     * Returns the tenant
     */
    public function getTenant(): int
    {
        return $this->tenantUid;
    }

    /**
     * Sets the tenant
     */
    public function setTenantById(?int $tenantId = null): void
    {
        $this->tenantUid = $tenantId;
    }
    /**
     * Sets the tenant
     */
    public function setTenantByUid(?int $tenantUid = null): void
    {
        $this->tenantUid = $tenantUid;
    }
    /**
     * Sets the tenant
     */
    public function setTenant(TenantInterface $tenant = null): void
    {
        $this->tenantUid = $tenant->getUid();
    }

    /**                                                                                                          
     * Returns the languageUid
     */
    public function getLanguageUid(): ?int
    {
        return $this->languageUid;
    }
    /**
     * Sets languageUid
     */
    public function setLanguageUid(?int $languageUid = null): void
    {
        $this->languageUid = $languageUid;
    }
    
    public function getTemplate(string $templateType, TemplateObjectInterface $templateObject = null, AddressInterface $contact = null): ?TemplateLetter
    {
        $objectType = null;
        if ($contact) {
            $objectType = $contact->getType();
        }
        if ($this->tenantUid) {
            CrmTranslate::setVerband($this->tenantUid);
        }
        $crmLanguageUid = $this->getLanguageUid();
        CrmTranslate::setCrmLanguage($crmLanguageUid);
        //die('U=' . $this->tenantUid);

        return $this->templateLetterRepository->findByTenantUidTypeWithTemplateObject($this->tenantUid, $templateType, $templateObject, $contact, $crmLanguageUid);
    }

    public function getDefaultTemplatePageLayout(string $pageFormat = 'A4', $pageOrientation = 'portrait', array $margins = ['15mm', '0', '0', '0']): TemplatePageLayout
    {
        $templatePageLayout = GeneralUtility::makeInstance(TemplatePageLayout::class);
        $templatePageLayout->setPageFormat($pageFormat);
        $templatePageLayout->setPageOrientation($pageOrientation);
        $templatePageLayout->setName('Default ' . $pageFormat .' ' . $pageOrientation);
        if (!empty($margins)) {
            $templatePageLayout->setPageMarginTop((string)$margins[0]);
            $templatePageLayout->setPageMarginRight((string)($margins[1] ?? $margins[0]));
            $templatePageLayout->setPageMarginBottom((string)($margins[2] ?? $margins[0]));
            $templatePageLayout->setPageMarginLeft((string)($margins[3] ?? $margins[1] ?? $margins[0]));
        }
        //var_dump($templatePageLayout);exit(0);
        return $templatePageLayout;
    }

    public function getTemplatePageLayout(string $templateType, string $pageFormat = 'A4', $pageOrientation = 'portrait', array $margins = ['15mm', '0', '0mm', '0']): ?TemplatePageLayout
    {
        $templatePageLayout = $this->templatePageLayoutRepository->findOneByTenantUidAndTemplateTypeName($this->tenantUid, $templateType);
        if ($templatePageLayout instanceof TemplatePageLayout) {
            return $templatePageLayout;
        }
        if ($pageFormat && $pageOrientation) {
            return $this->getDefaultTemplatePageLayout($pageFormat, $pageOrientation, $margins);
        }
        return  null;
    }
    
    public function renderCssPagedMediaMarginBoxs(TemplatePageLayout $templatePageLayout, RenderingContextInterface $renderingContext = null)
    {
        $cssPagedMediaMarginBoxs = $templatePageLayout->getCssPagedMediaMarginBoxs();
        if (!empty($cssPagedMediaMarginBoxs)) {
            $renderCssPagedMediaMarginBoxs = GeneralUtility::makeInstance(CssPagedMediaMarginBoxs::class);
            foreach($cssPagedMediaMarginBoxs as $cssPagedMediaMarginBox) {
                $renderCssPagedMediaMarginBoxs->add($cssPagedMediaMarginBox, $renderingContext);
            }
            return $renderCssPagedMediaMarginBoxs->getStyle() . $renderCssPagedMediaMarginBoxs->getHtml();
        }
        return '';
    }
    /*
    public function getPageLayoutCss()
    {
        return $this->getPageLayoutCssA4Portrait();
    }
    public function getPageLayoutCssA4Portrait()
    {
        return file_get_contents(GeneralUtility::getFileAbsFileName('EXT:ig_crm_template/Resources/Public/Css/PageLayout/a4-portrait.css'));

    }
    public function getPageLayoutCssA4Landscapt()
    {
        return file_get_contents(GeneralUtility::getFileAbsFileName('EXT:ig_crm_template/Resources/Public/Css/PageLayout/a4-landscape.css'));

    }
    */
    
}
